!!This subroutine is to calculate the electron density in different L and Latitude utilizing different density models

subroutine ElectronDensity(L,lambda,Ne_local)
use constants
implicit none

  real*8,intent(in)::L,lambda
  real*8,intent(out)::Ne_local

SELECT CASE(DenModel)
    CASE('Sheely01','SHEELY01','sheely01') 
         Ne_local=1.24e8*((3.0/L)**4.0)+3.6e7*((3.0/L)**3.5)*cos((MLT-(7.7*((3.0/L)**2.0)+12.0))*PI/12.0)   ! Sheeley et al. 2001 Equation 7, related with L and MLT
    CASE('CA92', 'Ca92', 'cA92', 'ca92') 
         Ne_local=1e6*10.0**(-0.3145*L+3.9043)  !Carpenter and Anderson 1992 Plasmaspheric Density Model, only related with L
    CASE('DENTON06', 'Denton06', 'denton06') 
         Ne_local=1e6*10.0**(-0.324*L+3.78+(0.00127*78.0-0.0635)*exp((2.0-L)/1.5));  !  Denton et al. 2004,2006 Density Model, only related with L
    CASE('CONSTANT', 'Constant', 'constant') 
         Ne_local=DenValue*1e6
    CASE('NbbLBUB')
        if(kp.eq.0)then
                Ne_local=16.9e6
        else if(kp.eq.1)then
                Ne_local=5.52e6
        else
                Ne_local=2.88e6
        endif
    CASE('XD_Saturn')
        Ne_local=60e6*exp(-1/3.0*L**2/0.735**2*(1.0-cos(lambda)**6))
    CASE DEFAULT
        WRITE(*,*) 'Invalid DENSITY MODEL.'
        STOP
    END SELECT
 

Ne_local=Ne_local*(cos(lambda))**DenLatIndex

  

  return

end subroutine ElectronDensity
